/* This is a -*- C++ -*- header file. */

/* flags.h -- Flags

   Copyright 2002-2004 Jesus A. De Loera, David Haws, Raymond
      Hemmecke, Peter Huggins, Jeremy Tauzer, Ruriko Yoshida
   Copyright 2006 Matthias Koeppe

   This file is part of LattE.
   
   LattE is free software; you can redistribute it and/or modify it
   under the terms of the version 2 of the GNU General Public License
   as published by the Free Software Foundation.

   LattE is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with LattE; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef FLAGS_H
#define FLAGS_H 1

#define PRINT 0x1
#define OUTPUT 0x6
#define OUTPUT0 0x2
#define OUTPUT1 0x4
#define DUAL_APPROACH 0x8
#define DECOMPOSE 0x10
#define LOAD	0x20
#define SAVE    0x40

#endif
